<?php
/**
 * \file Instance.class.inc
 * \class Instance
 *
 * \author Yoann Perollet <yoann.perollet@veremes.com>.
 *
 * 	\brief This file contains the Instance php class
 *
 * This class defines Rest Api to Gtf order statutes
 * 
 */
require_once 'Gtf.class.inc';
require_once __DIR__ . '/../../class/vitis_lib/Connection.class.inc';
require_once __DIR__ . '/../../class/vmlib/BdDataAccess.inc';

class Instance extends GTF {
    /**
     * @SWG\Definition(
     *   definition="/instance",
     *   allOf={
     *     @SWG\Schema(ref="#/definitions/instance")
     *   }
     * )
     * * @SWG\Tag(
     *   name="Instance",
     *   description="Operations about Instance"
     * )
     */

    /**
     * construct
     * @param type $aPath url of the request
     * @param type $aValues parameters of the request
     * @param type $properties properties
     * @param type $bShortcut false to reinit variables
     * @param type $oConnection connection object
     */
    function __construct($aPath, $aValues, $properties, $bShortcut = false, $oConnection = false) {
        parent::__construct($aPath, $aValues, $properties, $bShortcut, $oConnection);
        $this->aSelectedFields = Array('priority_id', 'priority_label');
    }

    /**
     * @SWG\Get(path="/instance/calculate",
     *   tags={"Instance"},
     *   summary="Get instance id",
     *   description="Request to get instance id",
     *   operationId="GET",
     *   produces={"application/xml", "application/json", "application/x-vm-json"},
     *   @SWG\Parameter(
     *     name="token",
     *     in="query",
     *     description="user token",
     *     required=true,
     *     type="string"
     *   ),
     *   @SWG\Response(
     *         response=200,
     *         description="Poprerties Response",
     *         @SWG\Schema(ref="#/definitions/instance")
     *     )
     *  )
     */

    /**
     * get Instance
     * @return  Instance
     */
    function GET() {
        if (!empty($this->aPath[2])) {
            if ($this->aPath[2] == 'calculate') {
            }
        }
        $this->calculateGtfInstanceId();
        $aXmlRacineAttribute['status'] = 1;
        $sMessage = $this->asDocument('', 'vitis', $this->aValues['sEncoding'], True, $aXmlRacineAttribute, $this->aValues['sSourceEncoding'], $this->aValues['output']);
        return $sMessage;
    }
    
    /**
     * Calculating a GTF instance id.
     */
    function calculateGtfInstanceId() {
        $aOutput = array();
        $iReturnStatus = 1;
        if (strtoupper(substr(PHP_OS, 0, 3)) === 'WIN') {
            // Paramètres à concaténer pour un identifiant unique par machine.
            $aCommands = array(
                'NICCONFIG',    // cartes réseaux (adresses MAC)
                'CPU',  // modèle, vitesse, id...
                'DISKDRIVE',    // N° de série...
                'MEMORYCHIP',
                'CDROM',    // Lecteur DVD...
                'path win32_VideoController',   // Carte vidéo
                'IDECONTROLLER',
                'SCSICONTROLLER',
                'DESKTOPMONITOR',
                'LOGON',
                'NETLOGIN'
            );
            $sCommandLine = 'wmic '. implode(' & wmic ', $aCommands);
        }
        exec($sCommandLine, $aOutput, $iReturnStatus);
        $this->aFields['instance_id'] = hash_hmac('sha256', implode(chr(20), $aOutput), uniqid('', true));
    }    
}
?>